import json
import os


def _get_default_path():
    return os.path.join(os.getcwd(), '.better_test.db')


def read_database(path=None):
    if path is None:
        path = _get_default_path()
    if os.path.exists(path):
        with open(path) as fobj:
            return json.load(fobj)
    else:
        return {}


def write_database(database, path=None):
    if path is None:
        path = _get_default_path()
    with open(path, 'w') as fobj:
        json.dump(database, fobj)



from __future__ import absolute_import
from optparse import make_option
import os
import multiprocessing
import itertools
import sys
import warnings

from django.core.management.commands.test import Command as DjangoTest
from django.conf import settings

from ...database import read_database
from ...utils import DisableMigrations
from ...utils import get_test_runner
from ...core import Config
from ...core import run
from ...core import ISOLATED
from ...core import PARALLEL
from ...core import STANDARD


SEPARATOR_1 = '=' * 70
SEPARATOR_2 = '-' * 70


def args_builder(factory, parallel=True):
    args = []
    if parallel:
        args.append(factory(
            '--parallel',
            action='store_true', dest='parallel', default=False,
            help='Run tests in parallel.'
        ))
    return args + [
        factory('--isolate',
                action='store_true', dest='isolate', default=False,
                help='Run each test isolated.'),
        factory('--failed',
                action='store_true', dest='failed', default=False,
                help='Re-run tests that failed the last time.'),
        factory('--list-slow',
                type=int, dest='list_slow', default=0,
                help='Amount of slow tests to print.'),
        factory('--retest',
                action='store_true', dest='retest', default=False,
                help='Re-run the tests using the last configuration.'),
        factory('--vanilla',
                action='store_true', dest='vanilla', default=False,
                help='Ignore better test.'),
        factory('--no-migrations',
                action='store_true', dest='no_migrate', default=False,
                help='Do not run migrations'),
        factory('--migrate',
                action='store_true', dest='migrate', default=False,
                help='Run migrations (slow)'),
        factory('--start-method', dest='start_method', default='spawn',
                help='Select multiprocessing spawn method',
                choices=['fork', 'spawn', 'forkserver'])
    ]


class Command(DjangoTest):
    if hasattr(DjangoTest, 'option_list'):
        option_list = DjangoTest.option_list + tuple(args_builder(make_option))

    def add_arguments(self, parser):
        super(Command, self).add_arguments(parser)
        args_builder(parser.add_argument, False)

    def handle(self, *test_labels, **options):
        from django.conf import settings

        if options['no_migrate']:
            warnings.warn(
                "The --no-migrations flag is deprecated. Migrations are "
                "skipped by default now, to use them, use --migrate.",
                DeprecationWarning
            )
            options['migrate'] = False

        if not options.pop('migrate'):
            settings.MIGRATION_MODULES = DisableMigrations()

        if options['vanilla']:
            return DjangoTest().handle(*test_labels, **options)
        else:
            database = read_database()
            test_runner_options = get_test_runner_options(options)
            test_labels, config = get_config(database, options, test_labels)
            patch_settings(options)
            result = run(test_labels, test_runner_options, config)
            display_result(self.stdout, result)
            if options['list_slow']:
                list_slow(self.stdout, result, options['list_slow'])
            save_result(result, database, options)
            sys.exit(result.total_failures)


def get_test_runner_options(options):
    """
    Build the options for the test runner class.
    """
    test_runner_options = dict(options.items())
    del test_runner_options['parallel']
    test_runner_options['verbosity'] = int(test_runner_options['verbosity'])
    test_runner_options['interactive'] = False
    return test_runner_options


def get_config(database, options, test_labels):
    """
    Turn the command options (and database info) into a Config object to be
    used by core.run.
    """
    test_runner = get_test_runner(options.get('testrunner'))

    if options['retest'] and database.get('last_run', None):
        last_run = database['last_run']
        if last_run['isolate']:
            mode = ISOLATED
        elif last_run['parallel']:
            mode = PARALLEL
        else:
            mode = STANDARD
        test_labels = last_run['labels']
    else:
        if options['isolate']:
            mode = ISOLATED
        elif options['parallel']:
            mode = PARALLEL
        else:
            mode = STANDARD

    return test_labels, Config(
        test_runner_class=test_runner,
        mode=mode,
        timings=database.get('timings', {}),
        processes=multiprocessing.cpu_count(),
        verbosity=int(options['verbosity']),
        start_method=options['start_method']
    )


def patch_settings(options):
    """
    Patch Django settings/environment.
    """
    if 'south' in settings.INSTALLED_APPS:
        from south.management.commands import patch_for_test_db_setup
        patch_for_test_db_setup()

    if options.get('liveserver') is not None:
        os.environ['DJANGO_LIVE_TEST_SERVER_ADDRESS'] = options['liveserver']


def _get_description(test):
    doc_first_line = test.shortDescription()
    if doc_first_line:
        return '\n'.join((str(test), doc_first_line))
    else:
        return str(test)


def _write_error_list(flavor, errors, writeln):
    for test, err in errors:
        writeln(SEPARATOR_1)
        writeln(
            '{flavor}: {description}'.format(
                flavor=flavor,
                description=_get_description(test)
            )
        )
        writeln(SEPARATOR_2)
        writeln(str(err))


def display_result(stream, result):
    """
    Write the result to the stream, mimicking unittest.
    """
    writeln = lambda s: stream.write('{0}\n'.format(s))
    writeln('')  # newline after the "dots"
    _write_error_list('ERROR', result.errors, writeln)
    _write_error_list('FAIL', result.failures, writeln)
    for chunk, exit_code in result.failed_executors:
        writeln(SEPARATOR_1)
        writeln('FAILED EXECUTOR: {exit_code}'.format(exit_code=exit_code))
        writeln(SEPARATOR_2)
        writeln(str(chunk))
    writeln(SEPARATOR_2)
    writeln(
        "Ran {number} test{plural} in {time:.3f}s".format(
            number=result.tests_run,
            plural=result.tests_run != 1 and "s" or "",
            time=result.time_taken
        )
    )
    writeln('')

    # Display info about failures etc
    infos = []
    skipped = len(result.skipped)
    expected_fails = len(result.expected_failures)
    unexpected_successes = len(result.unexpected_successes)
    failed_executors = len(result.failed_executors)
    if not result.success:
        stream.write("FAILED")
        failed = len(result.failures)
        errored = len(result.errors)
        if failed:
            infos.append("failures={failed}".format(failed=failed))
        if errored:
            infos.append("errors={errors}".format(errors=errored))
    elif result.failed_executors:
        stream.write("executors failed={failed_executors}".format(
            failed_executors=failed_executors
        ))
    else:
        stream.write("OK")
    if skipped:
        infos.append("skipped={skipped}".format(skipped=skipped))
    if expected_fails:
        infos.append("expected failures={expected_failures}".format(
            expected_failures=expected_fails
        ))
    if unexpected_successes:
        infos.append("unexpected successes={unexpected_successes}".format(
            unexpected_successes=unexpected_successes
        ))
    if infos:
        writeln(" ({info})".format(info=", ".join(infos)))
    else:
        writeln('')


def list_slow(stream, result, num):
    """
    List the `num` slowest tests.
    """
    writeln = lambda s: stream.write('{0}\n'.format(s))
    writeln("Slowest tests:")
    slowest = sorted(
        ((timing, test) for test, timing in result.timings.items()),
        reverse=True
    )
    for timing, test in slowest[:num]:
        writeln(" {timing:.3f}s: {test}".format(timing=timing, test=test))
    writeln('')


def save_result(result, database, options):
    """
    Persist the result and the options to the database.
    """
    # Record timings to database
    data = {
        'timings': database.get('timings', {}),
    }
    for test, timing in result.timings.items():
        data['timings'][test] = timing

    # Record failed tests to database
    data['failed'] = [
        test.qualname for test, _ in itertools.chain(
            result.failures,
            result.errors,
            [(test, None) for test in result.unexpected_successes]
        )
    ]

    # Record config to database
    data['last_run'] = {
        'isolate': options['isolate'],
        'parallel': options['parallel'],
        'list_slow': options['list_slow'],
        'labels': result.test_labels,
    }



from contextlib import contextmanager
import os
import sys
import itertools


class DisableMigrations(object):
    def __contains__(self, _):
        return True

    def __getitem__(self, _):
        return "notmigrations"


def simple_weighted_partition(weighted_data, partitions):
    results = [list() for _ in range(partitions)]
    sorted_data = reversed(sorted(weighted_data))
    index_iter = itertools.cycle(
        itertools.chain(range(partitions), reversed(range(partitions)))
    )
    for weight, value in sorted_data:
        results[next(index_iter)].append(value)
    return results


def suite_to_labels(suite, result):
    """
    Transform a unittest.TestSuite to a list of test labels that can be used
    by django.test.DiscoveryRunner.run_tests.
    """
    labels = []
    for test in suite._tests:
        klass = test.__class__
        name = klass.__name__
        module = klass.__module__
        if name == 'ModuleImportFailure' and module == 'unittest.loader':
            test.qualname = module + '.' + name
            try:
                getattr(test, test._testMethodName)()
            except Exception as err:
                result.addError(test, err)
        else:
            labels.append(test_to_dotted(test))
    return labels


@contextmanager
def null_stdout():
    """
    Context manager setting sys.stdout to devnull and yielding that stdout
    """
    stdout = sys.stdout
    nullout = open(os.devnull, 'w')
    sys.stdout = nullout
    try:
        yield nullout
    finally:
        sys.stdout = stdout
        nullout.close()


def serialize(test):
    """
    Serializes a test (which can either be a TestCase-like or an ErrorHolder)
    for safe transport via a Queue.
    ErrorHolder (if actual test setup failed, not anything during test run) is
    special cased because it's a dynamic class and isn't quite the same as
    normal tests.
    """
    if hasattr(test, '_testMethodName'):
        return (
            test_to_dotted(test),
            str(test),
            test.shortDescription()
        )
    else:
        return (
            test.id(),
            str(test),
            test.shortDescription(),
        )


def test_to_dotted(test):
    klass = test.__class__
    name = klass.__name__
    module = klass.__module__
    return '{module}.{name}.{method}'.format(
        module=module,
        name=name,
        method=test._testMethodName
    )


def get_test_runner(name=None):
    from django.conf import settings
    from django.test.utils import get_runner

    return get_runner(settings, name)


def get_settings_dict():
    from django.conf import settings
    return dict(
        (key, getattr(settings, key)) for key in dir(settings)
        if key.upper() == key
    )



from __future__ import absolute_import

from ..compat import unittest


class SkipTests(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        raise unittest.SkipTest("Skipping Class")

    def test_method(self):
        pass




from __future__ import absolute_import

from django.core.management.base import BaseCommand

from ...database import read_database


class Command(BaseCommand):
    def handle(self, *args, **options):
        database = read_database()
        if not database:
            self.stdout.write("No database found\n")
            return
        self.stdout.write("Last run test results\n")
        self.stdout.write("=====================\n\n")
        self.stdout.write("\n")
        self.stdout.write("Timings\n\n")
        for test, duration in sorted(database.get('timings', {}).items(),
                                     key=lambda x: -x[1]):
            self.stdout.write(
                "{duration:-7.3f} {test}\n".format(
                    test=test, duration=duration
                )
            )
        self.stdout.write("\n")
        self.stdout.write("Failed tests:\n\n")
        for failed in database.get('failed', []):
            self.stdout.write('    ' + failed + '\n')



from __future__ import absolute_import
import time

from .parallel import Pool
from .parallel import MultiProcessingTextTestResult
from .utils import suite_to_labels
from .utils import simple_weighted_partition
from .compat import unittest

ISOLATED = 1
PARALLEL = 2
STANDARD = 0


class Result(object):
    def __init__(self, tests_run, time_taken, timings, failures, errors,
                 skipped, expected_failures, unexpected_successes,
                 failed_executors, successes, test_labels):
        self.tests_run = tests_run
        self.time_taken = time_taken
        self.timings = timings
        self.failures = failures
        self.errors = errors
        self.skipped = skipped
        self.expected_failures = expected_failures
        self.unexpected_successes = unexpected_successes
        self.failed_executors = failed_executors
        self.successes = successes
        self.test_labels = test_labels

    @property
    def total_results(self):
        return sum(map(len, (
            self.failures,
            self.errors,
            self.skipped,
            self.expected_failures,
            self.unexpected_successes,
            self.failed_executors,
            self.successes,
        )))

    @property
    def success(self):
        return not any((
            self.unexpected_successes,
            self.failures,
            self.errors,
            self.failed_executors
        ))

    @property
    def total_failures(self):
        return sum(map(len, (
            self.failures,
            self.errors,
            self.unexpected_successes,
            self.expected_failures,
            self.failed_executors
        )))


class Config(object):
    def __init__(self, test_runner_class, mode, timings, processes,
                 verbosity=1, debug=False, start_method='spawn'):
        self.test_runner_class = test_runner_class
        self.mode = mode
        self.timings = timings
        self.processes = processes
        self.verbosity = verbosity
        self.debug = debug
        self.start_method = start_method


def run(test_labels, test_runner_options, config,
        real_result_class=MultiProcessingTextTestResult):
    test_runner = config.test_runner_class(**test_runner_options)

    suite = test_runner.build_suite(test_labels)

    # Get an actual result class we can use
    pseudo_runner = unittest.TextTestRunner(
        resultclass=real_result_class,
        verbosity=config.verbosity,
    )
    real_result = pseudo_runner._makeResult()

    all_test_labels = suite_to_labels(suite, real_result)

    if config.mode == ISOLATED:
        # Isolate means one test (label) per task process.
        chunks = [
            [label] for label in all_test_labels
        ]
    elif config.mode == PARALLEL:
        # Try to distribute the test labels equally across the available
        # CPU cores
        chunk_count = config.processes
        weighted_chunks = [
            (config.timings.get(label, 0), label)
            for label in all_test_labels
        ]
        # Try to split the tests into chunks of equal time, not equal size
        chunks = simple_weighted_partition(weighted_chunks, chunk_count)

        # filter empty chunks
        chunks = list(filter(bool, chunks))
    elif config.mode == STANDARD:
        chunks = [all_test_labels]
    else:
        raise ValueError("Unknown mode: {0}".format(config.mode))

    start_time = time.time()
    pool = Pool(real_result, config.processes, config.start_method)
    failed_executors = pool.run(
        chunks,
        config.test_runner_class,
        test_runner_options
    )
    end_time = time.time()

    # Report result, this is mostly taken from TextTestRunner.run
    time_taken = end_time - start_time

    timings = dict(real_result.timings.items())

    return Result(
        tests_run=real_result.testsRun,
        time_taken=time_taken,
        timings=timings,
        failures=real_result.failures,
        errors=real_result.errors,
        skipped=real_result.skipped,
        expected_failures=real_result.expectedFailures,
        unexpected_successes=real_result.unexpectedSuccesses,
        failed_executors=failed_executors,
        successes=real_result.successes,
        test_labels=all_test_labels,
    )



import sys
import multiprocessing

try:
    import queue
except ImportError:
    import Queue as queue


PY_26 = sys.version_info[0] == 2 and sys.version_info[1] == 6

if PY_26:
    from django.utils import unittest
else:
    import unittest


try:
    get_multiprocessing_context = multiprocessing.get_context
except AttributeError:
    get_multiprocessing_context = lambda method: multiprocessing



from __future__ import absolute_import

from ..compat import unittest


class IsolateTests(unittest.TestCase):
    state = 0

    def test_one(self):
        IsolateTests.state += 2
        self.assertEqual(IsolateTests.state, 2)

    def test_two(self):
        IsolateTests.state += 20
        self.assertEqual(IsolateTests.state, 20)



from __future__ import absolute_import

from ..compat import unittest


class Tests(unittest.TestCase):
    def test_success(self):
        self.assertTrue(True)

    def test_fail(self):
        self.assertTrue(False)

    def test_exception(self):
        raise Exception("Test")

    @unittest.expectedFailure
    def test_expected_failure(self):
        self.assertTrue(False)

    @unittest.expectedFailure
    def test_unexpected_success(self):
        self.assertTrue(True)

    def test_skip(self):
        raise unittest.SkipTest("Skipping (exc)")

    @unittest.skip("Skipping (deco)")
    def test_skip_deco(self):
        self.assertTrue(False)



import contextlib
import sys
import signal

from better_test.compat import unittest

from better_test import core
from better_test.parallel import SilentMultiProcessingTextTestResult
from better_test.utils import get_test_runner

if sys.platform == 'darwin':
    import subprocess

    @contextlib.contextmanager
    def no_report_crash():
        subprocess.check_call([
            'launchctl',
            'unload',
            '-w',
            '/System/Library/LaunchAgents/com.apple.ReportCrash.plist'
        ])
        try:
            yield
        finally:
            subprocess.check_call([
                'launchctl',
                'load',
                '-w',
                '/System/Library/LaunchAgents/com.apple.ReportCrash.plist'
            ])
elif sys.platform == 'win32':
    import ctypes
    SEM_FAILCRITICALERRORS = 0x0001
    SEM_NOGPFAULTERRORBOX = 0x0002

    @contextlib.contextmanager
    def no_report_crash():
        old_mode = ctypes.windll.kernel32.GetErrorMode()
        ctypes.windll.kernel32.SetErrorMode(
            SEM_FAILCRITICALERRORS | SEM_NOGPFAULTERRORBOX
        )
        try:
            yield
        finally:
            ctypes.windll.kernel32.SetErrorMode(old_mode)
else:
    @contextlib.contextmanager
    def no_report_crash():
        yield


class TestCrashingExecutors(unittest.TestCase):
    def test_segfault(self):
        with no_report_crash():
            result = core.run(
                ['better_test.harness.crash.Tests.test_segfault'],
                {},
                core.Config(
                    test_runner_class=get_test_runner(),
                    mode=core.PARALLEL,
                    timings={},
                    processes=1,
                    debug=True
                ),
                real_result_class=SilentMultiProcessingTextTestResult
            )
        self.assertFalse(result.success)
        self.assertEqual(len(result.failed_executors), 1)
        _, exit_code = result.failed_executors[0]
        if sys.platform == 'linux':
            self.assertEqual(exit_code, -signal.SIGSEGV)
        elif sys.platform == 'darwin':
            self.assertEqual(exit_code, -signal.SIGILL)
        elif sys.platform == 'win32':
            self.assertEqual(exit_code, 3221225477)




from __future__ import absolute_import

import time
import multiprocessing

from .compat import unittest
from .compat import PY_26
from .compat import get_multiprocessing_context
from .utils import null_stdout
from .utils import serialize
from .utils import get_settings_dict


try:
    from coverage.collector import Collector
    from coverage.control import coverage
    if Collector._collectors:
        def mixin_coverage(cls):
            original = cls._bootstrap
            class Process(cls):
                def _bootstrap(self):
                    cov = coverage(data_suffix=True)
                    cov.start()
                    try:
                        return original(self)
                    finally:
                        cov.stop()
                        cov.save()
            return Process
    else:
        mixin_coverage = lambda cls: cls
except ImportError:
    mixin_coverage = lambda cls: cls


class Pool(object):
    def __init__(self, real_result, max_processes=multiprocessing.cpu_count(),
                 start_method='spawn'):
        self.real_result = real_result
        self.max_processes = max_processes
        self.processes = []
        self.context = get_multiprocessing_context(start_method)
        self.results = self.context.Queue()
        self.failed_executors = []

    def run(self, chunks, runner_class, runner_options):
        settings_dict = get_settings_dict()
        chunk_num = 0
        while chunks:
            while len(self.processes) >= self.max_processes:
                self.handle_results()
            chunk = chunks.pop()
            process = mixin_coverage(self.context.Process)(
                target=executor,
                args=(
                    chunk,
                    runner_class,
                    runner_options,
                    chunk_num,
                    self.results,
                    settings_dict
                )
            )
            process.start()
            self.processes.append((process, chunk))
            chunk_num += 1

        while len(self.processes):
            self.handle_results()

        return self.failed_executors

    def handle_results(self):
        while not self.results.empty():
            result = self.results.get_nowait()
            self.handle_result(result)
        done = []
        for process, chunk in self.processes:
            if not process.is_alive():
                done.append((process, chunk))
                if process.exitcode != 0:
                    self.failed_executors.append((
                        chunk, process.exitcode
                    ))
        for process in done:
            self.processes.remove(process)

    def handle_result(self, result):
        method_name, args = result
        arglist = list(args)
        test_info = arglist.pop(0)
        fake_test = FakeTest.deserialize(test_info)
        method = getattr(self.real_result, method_name)
        method(fake_test, *arglist)


def multi_processing_runner_factory(stream, results):
    """
    Creates a test runner with the MultiProcessinTestResult result class and
    overwriting the output stream.
    """
    test_result_class = type(
        'MultiProcessingTestResult',
        (MultiProcessingTestResult, ),
        {'_results_queue': results}
    )
    
    def inner(*args, **kwargs):
        args = list(args)
        if len(args) > 0:
            args[0] = stream
        else:
            kwargs['stream'] = stream
        if len(args) > 5:
            args[5] = test_result_class
        else:
            kwargs['resultclass'] = test_result_class
        return unittest.TextTestRunner(*args, **kwargs)
    return inner


def executor(labels, runner_class, runner_options, chunk_num, results, conf):
    """
    Test runner inside the task process.
    """
    # We need to patch the db name in case we're in --parallel or --isolate
    # mode (or any other mode with more than one chunk). But if there's only
    # a single chunk, don't change the db name. Therefore we don't modify the
    # name for the first chunk (chunk_num=0).
    from django.conf import settings
    if not settings.configured:
        import django
        settings.configure(**conf)
        django.setup()
    if chunk_num:
        for config in settings.DATABASES.values():
            if config.get('NAME', None) != ':memory:':
                config['NAME'] += '_{num}'.format(num=chunk_num)
    try:
        with null_stdout() as nullout:
            real_runner_class = type(
                runner_class.__name__,
                (MultiProcessingTestRunner, runner_class),
                {'test_runner': multi_processing_runner_factory(
                    nullout, results
                )}
            )
            runner = real_runner_class(**runner_options)
            runner.run_tests(labels)
    except:
        import traceback
        traceback.print_exc()
        raise


class MultiProcessingTextTestResult(unittest.TextTestResult):
    """
    Thin wrapper around TextTestResult. Python tracebacks are not pickleable,
    so _exc_info_to_string is handled in MultiProcessingTestResult and the
    result sent to this as `err`.
    """
    def __init__(self, *args, **kwargs):
        super(MultiProcessingTextTestResult, self).__init__(*args, **kwargs)
        self.timings = {}
        self.successes = []

    def registerTiming(self, test, timing):
        self.timings[test.qualname] = timing

    def addSuccess(self, test):
        """
        The default result class doesn't store successes, so we do it ourselves
        """
        super(MultiProcessingTextTestResult, self).addSuccess(test)
        self.successes.append(test)

    def _exc_info_to_string(self, err, test):
        """
        Actual transformation from exception info to string happens in the
        executor, as exceptions can't be transferred through a Queue.
        """
        return err


class SilentMultiProcessingTextTestResult(MultiProcessingTextTestResult):
    def __init__(self, *args, **kwargs):
        super(SilentMultiProcessingTextTestResult, self).__init__(
            *args, **kwargs
        )
        self.dots = False
        self.showAll = False


class MultiProcessingTestResult(unittest.TestResult):
    """
    Result class for used by the task processes. Instead of printing/storing
    any information, feeds results into QUEUE.

    Important to note is that `test` is transformed into a tuple of
    `(str(test), test.shortDescription())` as test case instances are not
    pickleable. Also note that exceptions are transformed to strings in the
    task as they're not picklable.
    """
    separator1 = '=' * 70
    separator2 = '-' * 70

    def __init__(self, *args, **kwargs):
        if PY_26:
            super(MultiProcessingTestResult, self).__init__()
        else:
            super(MultiProcessingTestResult, self).__init__(*args, **kwargs)
        self._timings = {}

    def printErrors(self):
        pass

    def startTest(self, test):
        self._timings[test] = time.time()
        self._results_queue.put((
            'startTest', (
                serialize(test),
            )
        ))

    def _setupStdout(self):
        pass

    def startTestRun(self):
        pass

    def stopTest(self, test):
        self._results_queue.put((
            'registerTiming', (
                serialize(test),
                time.time() - self._timings[test],
            )
        ))

    def _restoreStdout(self):
        pass

    def stopTestRun(self):
        pass

    def addError(self, test, err):
        safe_err = self._exc_info_to_string(err, test)
        self._results_queue.put((
            'addError', (
                serialize(test),
                safe_err
            )
        ))

    def addFailure(self, test, err):
        safe_err = self._exc_info_to_string(err, test)
        self._results_queue.put((
            'addFailure', (
                serialize(test),
                safe_err
            )
        ))

    def addSuccess(self, test):
        self._results_queue.put((
            'addSuccess', (
                serialize(test),
            )
        ))

    def addSkip(self, test, reason=None):
        self._results_queue.put((
            'addSkip', (
                serialize(test),
                reason
            )
        ))

    def addExpectedFailure(self, test, err):
        safe_err = self._exc_info_to_string(err, test)
        self._results_queue.put((
            'addExpectedFailure', (
                serialize(test),
                safe_err
            )
        ))

    def addUnexpectedSuccess(self, test):
        self._results_queue.put((
            'addUnexpectedSuccess', (
                serialize(test),
            )
        ))

    def wasSuccessful(self):
        pass


class FakeTest(object):
    """
    Object faking to be a test case. All the test runner/result need are
    __str__ and shortDescription, so that's all that is provided.
    """
    def __init__(self, qualname, name, description):
        self.qualname = qualname
        self.name = name
        self.description = description

    @classmethod
    def deserialize(cls, data):
        return cls(*data)

    def __str__(self):
        return self.name

    def shortDescription(self):
        return self.description


class MultiProcessingTestRunner(object):
    test_runner = unittest.TextTestRunner

    def run_suite(self, suite, **_):
        """
        Backport from Django 1.7
        """
        return self.test_runner(
            verbosity=self.verbosity,
            failfast=self.failfast,
        ).run(suite)



from setuptools import setup, find_packages


setup(
    name='django-better-test',
    version='0.10',
    description='A better test command for Django',
    url='https://github.com/ojii/django-better-test',
    author='Jonas Obrist',
    author_email='ojiidotch@gmail.com',
    license='BSD',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
    packages=find_packages(),
    install_requires=[
        'Django>=1.7',
    ],
)



from better_test.compat import unittest

from better_test import core
from better_test.parallel import SilentMultiProcessingTextTestResult
from better_test.utils import get_test_runner


class IsolationTests(unittest.TestCase):
    def test_state_isolation(self):
        result = core.run(
            ['better_test.harness.isolate.IsolateTests'],
            {},
            core.Config(
                test_runner_class=get_test_runner(),
                mode=core.ISOLATED,
                timings={},
                processes=1,
                debug=True
            ),
            real_result_class=SilentMultiProcessingTextTestResult
        )
        self.assertTrue(result.success, (result.failures, result.errors))
        self.assertEqual(len(result.successes), 2)

    def test_state_leak(self):
        result = core.run(
            ['better_test.harness.isolate.IsolateTests'],
            {},
            core.Config(
                test_runner_class=get_test_runner(),
                mode=core.STANDARD,
                timings={},
                processes=1,
                debug=True
            ),
            real_result_class=SilentMultiProcessingTextTestResult
        )
        self.assertFalse(result.success, (result.failures, result.errors))
        self.assertEqual(len(result.successes), 1)
        self.assertEqual(len(result.failures), 1)
